@@ -665,7 +665,7 @@ save_comp (CompEditor *editor)
 			listen_for_changes (editor);
 		}
 
-		comp_editor_set_changed (editor, FALSE);
+		priv->changed = FALSE;
 		priv->saved = TRUE;
 	}
 
@@ -957,7 +957,7 @@ action_save_cb (GtkAction *action,
 		e_alert_submit (
 			E_ALERT_SINK (editor),
 			"calendar:prompt-read-only-cal-editor",
-			e_source_peek_name (
+			e_source_get_display_name (
 				e_client_get_source (E_CLIENT (priv->cal_client))),
 			NULL);
 		return;
@@ -969,7 +969,7 @@ action_save_cb (GtkAction *action,
 		e_alert_submit (
 			E_ALERT_SINK (editor),
 			"calendar:prompt-no-task-assignment-editor",
-			e_source_peek_name (
+			e_source_get_display_name (
 				e_client_get_source (E_CLIENT (priv->cal_client))),
 			NULL);
 		return;
@@ -996,8 +996,6 @@ action_save_cb (GtkAction *action,
 			(GtkWindow *) editor, priv->cal_client, priv->comp))
 			return;
 
-	gtk_widget_set_sensitive (GTK_WIDGET (editor), FALSE);
-
 	if (save_comp_with_send (editor)) {
 		CompEditorFlags flags;
 		gboolean delegate;
@@ -1030,8 +1028,6 @@ action_save_cb (GtkAction *action,
 	} else
 		correct = FALSE;
 
-	gtk_widget_set_sensitive (GTK_WIDGET (editor), TRUE);
-
 	if (correct)
 		close_dialog (editor);
 }
@@ -2188,7 +2184,7 @@ prompt_and_save_changes (CompEditor *editor,
 			e_alert_submit (
 				E_ALERT_SINK (editor),
 				"calendar:prompt-read-only-cal-editor",
-				e_source_peek_name (
+				e_source_get_display_name (
 					e_client_get_source (E_CLIENT (priv->cal_client))),
 				NULL);
 			/* don't discard changes when selected readonly calendar */
@@ -2201,7 +2197,7 @@ prompt_and_save_changes (CompEditor *editor,
 			e_alert_submit (
 				E_ALERT_SINK (editor),
 				"calendar:prompt-no-task-assignment-editor",
-				e_source_peek_name (
+				e_source_get_display_name (
 					e_client_get_source (E_CLIENT (priv->cal_client))),
 				NULL);
 			return FALSE;
@@ -3129,7 +3125,7 @@ real_edit_comp (CompEditor *editor,
 
 	fill_widgets (editor);
 
-	comp_editor_set_changed (editor, FALSE);
+	priv->changed =FALSE;
 
 	listen_for_changes (editor);
 }
