@@ -494,8 +494,8 @@ search_address_in_addressbooks (const gchar *address,
 		const gchar *uid;
 		GError *err = NULL;
 
-		uid = e_source_peek_uid (source);
-		display_name = e_source_peek_name (source);
+		uid = e_source_get_uid (source);
+		display_name = e_source_get_display_name (source);
 
 		/* failed to load this book last time, skip it now */
 		if (g_hash_table_lookup (emu_broken_books_hash, uid) != NULL) {
@@ -751,19 +751,9 @@ em_utils_contact_photo (CamelInternetAddress *cia,
 				(const gchar *) photo->data.inlined.data,
 				photo->data.inlined.length, "image/jpeg");
 		} else {
-			gchar *filename = g_filename_from_uri (photo->data.uri, NULL, NULL);
-			gchar *data;
-			gsize len;
-
-			if (g_file_get_contents (filename, &data, &len, NULL)) {
-				camel_mime_part_set_content (part, data, len, "image/jpeg");
-				g_free (data);
-			} else {
-				g_object_unref (part);
-				part = NULL;
-			}
-
-			g_free (filename);
+			gchar *s = g_filename_from_uri (photo->data.uri, NULL, NULL);
+			camel_mime_part_set_filename (part, s);
+			g_free (s);
 		}
 	}
 
@@ -1092,7 +1082,7 @@ em_utils_is_local_delivery_mbox_file (CamelURL *url)
 {
 	g_return_val_if_fail (url != NULL, FALSE);
 
-        return g_str_equal (url->protocol, "mbox") &&
+	return g_str_equal (url->protocol, "mbox") &&
 		(url->path != NULL) &&
 		g_file_test (url->path, G_FILE_TEST_EXISTS) &&
 		!g_file_test (url->path, G_FILE_TEST_IS_DIR);
