@@ -481,6 +481,10 @@ mail_account_store_service_removed (EMailAccountStore *store,
 	account = e_get_account_by_uid (uid);
 	g_return_if_fail (account != NULL);
 
+	/* no change */
+	if (!account->enabled)
+		return;
+
 	provider = camel_service_get_provider (service);
 	g_return_if_fail (provider != NULL);
 
@@ -1148,7 +1152,7 @@ e_mail_account_store_remove_service (EMailAccountStore *store,
                                      CamelService *service)
 {
 	GtkTreeIter iter;
-	gboolean proceed = TRUE;
+	gboolean proceed;
 
 	g_return_if_fail (E_IS_MAIL_ACCOUNT_STORE (store));
 	g_return_if_fail (CAMEL_IS_SERVICE (service));
