@@ -73,6 +73,43 @@
 
 /* 1.4 upgrade functions */
 
+#define EM_TYPE_MIGRATE_SESSION \
+	(em_migrate_session_get_type ())
+
+typedef struct _EMMigrateSession {
+	CamelSession parent_object;
+
+	CamelStore *store;   /* new folder tree store */
+	gchar *srcdir;        /* old folder tree path */
+} EMMigrateSession;
+
+typedef struct _EMMigrateSessionClass {
+	CamelSessionClass parent_class;
+
+} EMMigrateSessionClass;
+
+GType em_migrate_session_get_type (void);
+
+G_DEFINE_TYPE (EMMigrateSession, em_migrate_session, CAMEL_TYPE_SESSION)
+
+static void
+em_migrate_session_class_init (EMMigrateSessionClass *class)
+{
+}
+
+static void
+em_migrate_session_init (EMMigrateSession *session)
+{
+}
+
+static EMMigrateSession *
+em_migrate_session_new (const gchar *user_data_dir)
+{
+	return g_object_new (
+		EM_TYPE_MIGRATE_SESSION,
+		"user-data-dir", user_data_dir, NULL);
+}
+
 static GtkProgressBar *progress;
 
 static void
@@ -429,6 +466,372 @@ em_update_sa_junk_setting_2_23 (void)
 	g_object_unref (client);
 }
 
+static gboolean
+mbox_to_maildir_migration_needed (EShellBackend *shell_backend)
+{
+	gchar *local_store;
+	gchar *local_outbox;
+	const gchar *data_dir;
+	gboolean migration_needed = FALSE;
+
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+
+	local_store = g_build_filename (data_dir, "local", NULL);
+	local_outbox = g_build_filename (local_store, ".Outbox", NULL);
+
+	/* If this is a fresh install (no local store exists yet)
+	 * then obviously there's nothing to migrate to Maildir. */
+	if (!g_file_test (local_store, G_FILE_TEST_IS_DIR))
+		migration_needed = FALSE;
+
+	/* Look for a Maildir Outbox folder. */
+	else if (!g_file_test (local_outbox, G_FILE_TEST_IS_DIR))
+		migration_needed = TRUE;
+
+	g_free (local_store);
+	g_free (local_outbox);
+
+	return migration_needed;
+}
+
+/* Folder names with '.' are converted to '_' */
+static gchar *
+sanitize_maildir_folder_name (gchar *folder_name)
+{
+	gchar *maildir_folder_name;
+
+	maildir_folder_name = g_strdup (folder_name);
+	g_strdelimit (maildir_folder_name, ".", '_');
+
+	 return maildir_folder_name;
+}
+
+static void
+copy_folder (CamelStore *mbox_store,
+             CamelStore *maildir_store,
+             const gchar *mbox_fname,
+             const gchar *maildir_fname)
+{
+	CamelFolder *fromfolder, *tofolder;
+	GPtrArray *uids;
+
+	fromfolder = camel_store_get_folder_sync (
+		mbox_store, mbox_fname, 0, NULL, NULL);
+	if (fromfolder == NULL) {
+		g_warning ("Cannot find mbox folder %s \n", mbox_fname);
+		return;
+	}
+
+	tofolder = camel_store_get_folder_sync (
+		maildir_store, maildir_fname,
+		CAMEL_STORE_FOLDER_CREATE,
+		NULL, NULL);
+	if (tofolder == NULL) {
+		g_warning ("Cannot create maildir folder %s \n", maildir_fname);
+		g_object_unref (fromfolder);
+		return;
+	}
+
+	uids = camel_folder_get_uids (fromfolder);
+	camel_folder_transfer_messages_to_sync (
+			fromfolder, uids, tofolder,
+			FALSE, NULL,
+			NULL, NULL);
+	camel_folder_free_uids (fromfolder, uids);
+
+	g_object_unref (fromfolder);
+	g_object_unref (tofolder);
+}
+
+static void
+copy_folders (CamelStore *mbox_store,
+              CamelStore *maildir_store,
+              CamelFolderInfo *fi,
+              EMMigrateSession *session)
+{
+	if (fi) {
+		if (!g_str_has_prefix (fi->full_name, ".#evolution")) {
+			gchar *maildir_folder_name;
+
+			/* sanitize folder names and copy folders */
+			maildir_folder_name = sanitize_maildir_folder_name (fi->full_name);
+			copy_folder (
+				mbox_store, maildir_store,
+				fi->full_name, maildir_folder_name);
+			g_free (maildir_folder_name);
+		}
+
+		if (fi->child)
+			copy_folders (mbox_store, maildir_store, fi->child, session);
+
+		copy_folders (mbox_store, maildir_store, fi->next, session);
+	}
+}
+
+struct MigrateStore {
+	EMMigrateSession *session;
+	CamelStore *mbox_store;
+	CamelStore *maildir_store;
+	gboolean complete;
+};
+
+static void
+migrate_stores (struct MigrateStore *ms)
+{
+	CamelFolderInfo *mbox_fi;
+	CamelStore *mbox_store = ms->mbox_store;
+	CamelStore *maildir_store = ms->maildir_store;
+
+	mbox_fi = camel_store_get_folder_info_sync (
+		mbox_store, NULL,
+		CAMEL_STORE_FOLDER_INFO_RECURSIVE |
+		CAMEL_STORE_FOLDER_INFO_FAST |
+		CAMEL_STORE_FOLDER_INFO_SUBSCRIBED,
+		NULL, NULL);
+
+	/* FIXME progres dialog */
+	copy_folders (mbox_store, maildir_store, mbox_fi, ms->session);
+	ms->complete = TRUE;
+
+	return;
+}
+
+static gboolean
+migrate_mbox_to_maildir (EShellBackend *shell_backend,
+                         EMMigrateSession *session)
+{
+	CamelService *mbox_service, *maildir_service;
+	CamelStore *mbox_store, *maildir_store;
+	CamelSettings *settings;
+	const gchar *data_dir;
+	gchar *path;
+	struct MigrateStore ms;
+
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+
+	mbox_service = camel_session_add_service (
+		CAMEL_SESSION (session), "local_mbox", "mbox",
+		CAMEL_PROVIDER_STORE, NULL);
+
+	settings = camel_service_get_settings (mbox_service);
+	path = g_build_filename (data_dir, "local_mbox", NULL);
+	g_object_set (settings, "path", path, NULL);
+	g_free (path);
+
+	maildir_service = camel_session_add_service (
+		CAMEL_SESSION (session), "local", "maildir",
+		CAMEL_PROVIDER_STORE, NULL);
+
+	settings = camel_service_get_settings (maildir_service);
+	path = g_build_filename (data_dir, "local", NULL);
+	g_object_set (settings, "path", path, NULL);
+	g_mkdir (path, 0700);
+	g_free (path);
+
+	mbox_store = CAMEL_STORE (mbox_service);
+	maildir_store = CAMEL_STORE (maildir_service);
+
+	ms.mbox_store = mbox_store;
+	ms.maildir_store = maildir_store;
+	ms.session = session;
+	ms.complete = FALSE;
+
+	g_thread_create ((GThreadFunc) migrate_stores, &ms, TRUE, NULL);
+	while (!ms.complete)
+		g_main_context_iteration (NULL, TRUE);
+
+	return TRUE;
+}
+
+static void
+rename_mbox_dir (EShellBackend *shell_backend)
+{
+	gchar *local_mbox_path, *new_mbox_path;
+	const gchar *data_dir;
+
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+	local_mbox_path = g_build_filename (data_dir, "local", NULL);
+	new_mbox_path = g_build_filename (data_dir, "local_mbox", NULL);
+
+	if (!g_file_test (local_mbox_path, G_FILE_TEST_EXISTS))
+		goto exit;
+
+	if (g_file_test (new_mbox_path, G_FILE_TEST_EXISTS))
+		goto exit;
+
+	g_rename (local_mbox_path, new_mbox_path);
+
+exit:
+	g_free (local_mbox_path);
+	g_free (new_mbox_path);
+}
+
+static gboolean
+create_mbox_account (EShellBackend *shell_backend,
+                     EMMigrateSession *session)
+{
+	EMailBackend *mail_backend;
+	EMailSession *mail_session;
+	CamelService *service;
+	CamelURL *url;
+	EAccountList *accounts;
+	EAccount *account;
+	const gchar *data_dir;
+	gchar *name, *id, *temp, *uri, *folder_uri;
+
+	mail_backend = E_MAIL_BACKEND (shell_backend);
+	mail_session = e_mail_backend_get_session (mail_backend);
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+
+	account = e_account_new ();
+	account->enabled = TRUE;
+
+	g_free (account->uid);
+	account->uid = g_strdup ("local_mbox");
+
+	url = camel_url_new ("mbox:", NULL);
+	temp = g_build_filename (data_dir, "local_mbox", NULL);
+	camel_url_set_path (url, temp);
+	g_free (temp);
+
+	uri = camel_url_to_string (url, 0);
+	e_account_set_string (account, E_ACCOUNT_SOURCE_URL, uri);
+
+#ifndef G_OS_WIN32
+	name = g_locale_to_utf8 (g_get_user_name (), -1, NULL, NULL, NULL);
+#else
+	name = g_strdup (g_get_user_name ());
+#endif
+
+	id = g_strconcat (name, "@", "localhost", NULL);
+	e_account_set_string (account, E_ACCOUNT_ID_NAME, name);
+	e_account_set_string (account, E_ACCOUNT_ID_ADDRESS, id);
+	e_account_set_string (account, E_ACCOUNT_NAME, id);
+
+	accounts = e_get_account_list ();
+	if (e_account_list_find (accounts, E_ACCOUNT_ID_ADDRESS, id)) {
+		g_object_unref (account);
+		goto exit;
+	}
+
+	/* This will also add it to the EMailSession. */
+	e_account_list_add (accounts, account);
+
+	service = camel_session_get_service (
+		CAMEL_SESSION (mail_session), account->uid);
+
+	folder_uri = e_mail_folder_uri_build (
+		CAMEL_STORE (service), "Sent");
+	e_account_set_string (
+		account, E_ACCOUNT_SENT_FOLDER_URI, folder_uri);
+	g_free (folder_uri);
+
+	folder_uri = e_mail_folder_uri_build (
+		CAMEL_STORE (service), "Drafts");
+	e_account_set_string (
+		account, E_ACCOUNT_DRAFTS_FOLDER_URI, folder_uri);
+	g_free (folder_uri);
+
+	e_account_list_save (accounts);
+
+exit:
+	camel_url_free (url);
+	g_free (uri);
+	g_free (name);
+	g_free (id);
+
+	return TRUE;
+}
+
+static void
+change_sent_and_drafts_local_folders (EShellBackend *shell_backend)
+{
+	EMailBackend *backend;
+	EMailSession *session;
+	EAccountList *accounts;
+	EIterator *iter;
+	const gchar *data_dir;
+	gboolean changed = FALSE;
+	CamelURL *url;
+	gchar *tmp_uri, *drafts_uri, *sent_uri, *old_drafts_uri, *old_sent_uri;
+
+	accounts = e_get_account_list ();
+	if (!accounts)
+		return;
+
+	backend = E_MAIL_BACKEND (shell_backend);
+	session = e_mail_backend_get_session (backend);
+
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+
+	tmp_uri = g_strconcat ("mbox:", data_dir, "/", "local", NULL);
+	url = camel_url_new (tmp_uri, NULL);
+	g_free (tmp_uri);
+
+	g_return_if_fail (url != NULL);
+
+	camel_url_set_fragment (url, "Drafts");
+	drafts_uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+
+	camel_url_set_fragment (url, "Sent");
+	sent_uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+
+	camel_url_free (url);
+
+	tmp_uri = g_strconcat (
+		"mbox:", g_get_home_dir (),
+		"/.evolution/mail/local", NULL);
+	url = camel_url_new (tmp_uri, NULL);
+	g_free (tmp_uri);
+
+	g_return_if_fail (url != NULL);
+
+	camel_url_set_fragment (url, "Drafts");
+	old_drafts_uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+
+	camel_url_set_fragment (url, "Sent");
+	old_sent_uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+
+	camel_url_free (url);
+
+	for (iter = e_list_get_iterator ((EList *) accounts);
+	     e_iterator_is_valid (iter); e_iterator_next (iter)) {
+		EAccount *account = (EAccount *) e_iterator_get (iter);
+		const gchar *uri;
+
+		if (!account)
+			continue;
+
+		uri = e_account_get_string (account, E_ACCOUNT_DRAFTS_FOLDER_URI);
+		if (g_strcmp0 (uri, drafts_uri) == 0 ||
+		    g_strcmp0 (uri, old_drafts_uri) == 0) {
+			changed = TRUE;
+			e_account_set_string (
+				account, E_ACCOUNT_DRAFTS_FOLDER_URI,
+				e_mail_session_get_local_folder_uri (
+				session, E_MAIL_LOCAL_FOLDER_DRAFTS));
+		}
+
+		uri = e_account_get_string (account, E_ACCOUNT_SENT_FOLDER_URI);
+		if (g_strcmp0 (uri, sent_uri) == 0 || g_strcmp0 (uri, old_sent_uri) == 0) {
+			changed = TRUE;
+			e_account_set_string (
+				account, E_ACCOUNT_SENT_FOLDER_URI,
+				e_mail_session_get_local_folder_uri (
+				session, E_MAIL_LOCAL_FOLDER_SENT));
+		}
+	}
+
+	g_object_unref (iter);
+	g_free (old_drafts_uri);
+	g_free (drafts_uri);
+	g_free (old_sent_uri);
+	g_free (sent_uri);
+
+	if (changed)
+		e_account_list_save (accounts);
+}
+
 static void
 em_rename_camel_url_params (CamelURL *url)
 {
@@ -439,7 +842,6 @@ em_rename_camel_url_params (CamelURL *url)
 		const gchar *url_parameter;
 		const gchar *property_name;
 	} camel_url_conversion[] = {
-		{ "account_uid",		"account-uid" },
 		{ "ad_auth",			"gc-auth-method" },
 		{ "ad_browse",			"gc-allow-browse" },
 		{ "ad_expand_groups",		"gc-expand-groups" },
@@ -461,8 +863,6 @@ em_rename_camel_url_params (CamelURL *url)
 		{ "folder_hierarchy_relative",	"folder-hierarchy-relative" },
 		{ "imap_custom_headers",	"fetch-headers-extra" },
 		{ "keep_on_server",		"keep-on-server" },
-		{ "oab_offline",		"oab-offline" },
-		{ "oal_selected",		"oal-selected" },
 		{ "offline_sync",		"stay-synchronized" },
 		{ "override_namespace",		"use-namespace" },
 		{ "owa_path",			"owa-path" },
@@ -669,6 +1069,45 @@ em_rename_account_params (void)
 	e_account_list_save (account_list);
 }
 
+static gboolean
+migrate_local_store (EShellBackend *shell_backend)
+{
+	EMMigrateSession *session;
+	const gchar *data_dir;
+	gchar *local_store;
+	gint response;
+
+	if (!mbox_to_maildir_migration_needed (shell_backend))
+		return TRUE;
+
+	response = e_alert_run_dialog_for_args (
+		e_shell_get_active_window (NULL),
+		"mail:ask-migrate-store", NULL);
+
+	if (response == GTK_RESPONSE_CANCEL)
+		exit (EXIT_SUCCESS);
+
+	rename_mbox_dir (shell_backend);
+	data_dir = e_shell_backend_get_data_dir (shell_backend);
+	local_store = g_build_filename (data_dir, "local", NULL);
+
+	if (!g_file_test (local_store, G_FILE_TEST_EXISTS))
+		g_mkdir_with_parents (local_store, 0700);
+
+	session = em_migrate_session_new (data_dir);
+	camel_session_set_online (CAMEL_SESSION (session), FALSE);
+
+	migrate_mbox_to_maildir (shell_backend, session);
+	create_mbox_account (shell_backend, session);
+	change_sent_and_drafts_local_folders (shell_backend);
+
+	g_object_unref (session);
+
+	g_free (local_store);
+
+	return TRUE;
+}
+
 static void
 em_ensure_proxy_ignore_hosts_being_list (void)
 {
@@ -861,6 +1300,9 @@ e_mail_migrate (EShellBackend *shell_backend,
 	 * match CamelSettings property names. */
 	em_rename_account_params ();
 
+	if (!migrate_local_store (shell_backend))
+		return FALSE;
+
 	if (major <= 2 || (major == 3 && minor < 4))
 		em_rename_folder_views (shell_backend);
 
