@@ -59,8 +59,9 @@ struct _EMailPanedViewPrivate {
 	GtkWidget *scrolled_window;
 	GtkWidget *message_list;
 	GtkWidget *preview_pane;
+	GtkWidget *search_bar;
 
-	EMFormatHTMLDisplay *formatter;
+	EMailDisplay *display;
 	GalViewInstance *view_instance;
 
 	/* ETable scrolling hack */
@@ -83,7 +84,6 @@ enum {
 #define STATE_KEY_GROUP_BY_THREADS	"GroupByThreads"
 #define STATE_KEY_SELECTED_MESSAGE	"SelectedMessage"
 #define STATE_KEY_PREVIEW_VISIBLE	"PreviewVisible"
-#define STATE_GROUP_GLOBAL_FOLDER	"GlobalFolder"
 
 /* Forward Declarations */
 static void e_mail_paned_view_reader_init (EMailReaderInterface *interface);
@@ -122,8 +122,6 @@ mail_paned_view_save_boolean (EMailView *view,
 	g_free (group_name);
 	g_free (folder_uri);
 
-	g_key_file_set_boolean (key_file, STATE_GROUP_GLOBAL_FOLDER, key, value);
-
 	e_shell_view_set_state_dirty (shell_view);
 }
 
@@ -361,11 +359,6 @@ mail_paned_view_dispose (GObject *object)
 		priv->preview_pane = NULL;
 	}
 
-	if (priv->formatter != NULL) {
-		g_object_unref (priv->formatter);
-		priv->formatter = NULL;
-	}
-
 	if (priv->view_instance != NULL) {
 		g_object_unref (priv->view_instance);
 		priv->view_instance = NULL;
@@ -430,14 +423,14 @@ mail_paned_view_get_backend (EMailReader *reader)
 	return E_MAIL_BACKEND (shell_backend);
 }
 
-static EMFormatHTML *
-mail_paned_view_get_formatter (EMailReader *reader)
+static EMailDisplay *
+mail_paned_view_get_mail_display (EMailReader *reader)
 {
-	EMailPanedView *paned_view;
+	EMailPanedViewPrivate *priv;
 
-	paned_view = E_MAIL_PANED_VIEW (reader);
+	priv = E_MAIL_PANED_VIEW (reader)->priv;
 
-	return EM_FORMAT_HTML (paned_view->priv->formatter);
+	return priv->display;
 }
 
 static gboolean
@@ -515,7 +508,7 @@ mail_paned_view_set_folder (EMailReader *reader,
 	gchar *folder_uri;
 	gchar *group_name;
 	const gchar *key;
-	gboolean value, global_view_settings;
+	gboolean value;
 	GError *error = NULL;
 
 	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (reader);
@@ -526,7 +519,6 @@ mail_paned_view_set_folder (EMailReader *reader,
 
 	shell = e_shell_window_get_shell (shell_window);
 	shell_settings = e_shell_get_shell_settings (shell);
-	global_view_settings = e_shell_settings_get_boolean (shell_settings, "mail-global-view-setting");
 
 	message_list = e_mail_reader_get_message_list (reader);
 
@@ -559,31 +551,19 @@ mail_paned_view_set_folder (EMailReader *reader,
 	g_free (folder_uri);
 
 	key = STATE_KEY_GROUP_BY_THREADS;
-	value = g_key_file_get_boolean (key_file, global_view_settings ? STATE_GROUP_GLOBAL_FOLDER : group_name, key, &error);
+	value = g_key_file_get_boolean (key_file, group_name, key, &error);
 	if (error != NULL) {
+		value = TRUE;
 		g_clear_error (&error);
-
-		value = !global_view_settings ||
-			g_key_file_get_boolean (key_file, STATE_GROUP_GLOBAL_FOLDER, key, &error);
-		if (error != NULL) {
-			g_clear_error (&error);
-			value = TRUE;
-		}
 	}
 
 	e_mail_reader_set_group_by_threads (reader, value);
 
 	key = STATE_KEY_PREVIEW_VISIBLE;
-	value = g_key_file_get_boolean (key_file, global_view_settings ? STATE_GROUP_GLOBAL_FOLDER : group_name, key, &error);
+	value = g_key_file_get_boolean (key_file, group_name, key, &error);
 	if (error != NULL) {
+		value = TRUE;
 		g_clear_error (&error);
-
-		value = !global_view_settings ||
-			g_key_file_get_boolean (key_file, STATE_GROUP_GLOBAL_FOLDER, key, &error);
-		if (error != NULL) {
-			g_clear_error (&error);
-			value = TRUE;
-		}
 	}
 
 	/* XXX This is a little confusing and needs rethought.  The
@@ -638,7 +618,6 @@ mail_paned_view_constructed (GObject *object)
 	EShellView *shell_view;
 	EShell *shell;
 	EShellSettings *shell_settings;
-	ESearchBar *search_bar;
 	EMailReader *reader;
 	EMailBackend *backend;
 	EMailSession *session;
@@ -646,10 +625,11 @@ mail_paned_view_constructed (GObject *object)
 	GtkWidget *message_list;
 	GtkWidget *container;
 	GtkWidget *widget;
-	EWebView *web_view;
 
 	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (object);
-	priv->formatter = em_format_html_display_new ();
+
+	priv->display = g_object_new (E_TYPE_MAIL_DISPLAY,
+		"headers-collapsable", TRUE, NULL);
 
 	view = E_MAIL_VIEW (object);
 	shell_view = e_mail_view_get_shell_view (view);
@@ -661,17 +641,9 @@ mail_paned_view_constructed (GObject *object)
 	backend = E_MAIL_BACKEND (shell_backend);
 	session = e_mail_backend_get_session (backend);
 
-	/* Make headers collapsable and store state of headers in config file */
-	em_format_html_set_headers_collapsable (
-		EM_FORMAT_HTML (priv->formatter), TRUE);
-	g_object_bind_property (
-		shell_settings, "paned-view-headers-state",
-		priv->formatter, "headers-state",
-		G_BINDING_BIDIRECTIONAL |
-		G_BINDING_SYNC_CREATE);
-
-	web_view = em_format_html_get_web_view (
-		EM_FORMAT_HTML (priv->formatter));
+	g_object_bind_property (shell_settings, "paned-view-headers-state",
+				priv->display, "headers-collapsed",
+				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
 	/* Build content widgets. */
 
@@ -708,11 +680,10 @@ mail_paned_view_constructed (GObject *object)
 
 	container = priv->paned;
 
-	gtk_widget_show (GTK_WIDGET (web_view));
-
-	widget = e_preview_pane_new (web_view);
+	widget = e_preview_pane_new (E_WEB_VIEW (priv->display));
 	gtk_paned_pack2 (GTK_PANED (container), widget, FALSE, FALSE);
 	priv->preview_pane = g_object_ref (widget);
+	gtk_widget_show (GTK_WIDGET (priv->display));
 	gtk_widget_show (widget);
 
 	g_object_bind_property (
@@ -720,12 +691,6 @@ mail_paned_view_constructed (GObject *object)
 		widget, "visible",
 		G_BINDING_SYNC_CREATE);
 
-	search_bar = e_preview_pane_get_search_bar (E_PREVIEW_PANE (widget));
-
-	g_signal_connect_swapped (
-		search_bar, "changed",
-		G_CALLBACK (em_format_queue_redraw), priv->formatter);
-
 	/* Load the view instance. */
 
 	e_mail_view_update_view_instance (E_MAIL_VIEW (object));
@@ -749,8 +714,6 @@ mail_paned_view_constructed (GObject *object)
 		G_CALLBACK (mail_paned_view_restore_state_cb),
 		object);
 
-	e_mail_reader_connect_headers (reader);
-
 	/* Do this after creating the message list.  Our
 	 * set_preview_visible() method relies on it. */
 	e_mail_view_set_preview_visible (view, TRUE);
@@ -765,26 +728,23 @@ static void
 mail_paned_view_set_search_strings (EMailView *view,
                                     GSList *search_strings)
 {
+	EMailDisplay *display;
+	EWebView *web_view;
 	EMailReader *reader;
-	EPreviewPane *preview_pane;
-	ESearchBar *search_bar;
-	ESearchingTokenizer *tokenizer;
 
 	reader = E_MAIL_READER (view);
-	preview_pane = e_mail_reader_get_preview_pane (reader);
-	search_bar = e_preview_pane_get_search_bar (preview_pane);
-	tokenizer = e_search_bar_get_tokenizer (search_bar);
+	display = e_mail_reader_get_mail_display (reader);
+	if (!display)
+		return;
 
-	e_searching_tokenizer_set_secondary_case_sensitivity (tokenizer, FALSE);
-	e_searching_tokenizer_set_secondary_search_string (tokenizer, NULL);
+	web_view = E_WEB_VIEW (display);
+
+	e_web_view_clear_highlights (web_view);
 
 	while (search_strings != NULL) {
-		e_searching_tokenizer_add_secondary_search_string (
-			tokenizer, search_strings->data);
+		e_web_view_add_highlight (web_view, search_strings->data);
 		search_strings = g_slist_next (search_strings);
 	}
-
-	e_search_bar_changed (search_bar);
 }
 
 static GalViewInstance *
@@ -1063,7 +1023,7 @@ e_mail_paned_view_reader_init (EMailReaderInterface *interface)
 	interface->get_action_group = mail_paned_view_get_action_group;
 	interface->get_alert_sink = mail_paned_view_get_alert_sink;
 	interface->get_backend = mail_paned_view_get_backend;
-	interface->get_formatter = mail_paned_view_get_formatter;
+	interface->get_mail_display = mail_paned_view_get_mail_display;
 	interface->get_hide_deleted = mail_paned_view_get_hide_deleted;
 	interface->get_message_list = mail_paned_view_get_message_list;
 	interface->get_popup_menu = mail_paned_view_get_popup_menu;
@@ -1108,6 +1068,14 @@ e_mail_paned_view_hide_message_list_pane (EMailPanedView *view,
 		gtk_widget_hide (view->priv->scrolled_window);
 }
 
+GtkWidget *
+e_mail_paned_view_get_preview (EMailPanedView *view)
+{
+	g_return_val_if_fail (E_IS_MAIL_PANED_VIEW (view), NULL);
+
+	return GTK_WIDGET (mail_paned_view_get_mail_display (E_MAIL_READER (view)));
+}
+
 void
 e_mail_paned_view_set_enable_show_folder (EMailPanedView *view,
                                           gboolean set)
