@@ -1907,12 +1907,13 @@ tree_drag_data_get (GtkWidget *widget,
                     EMFolderTree *folder_tree)
 {
 	EMFolderTreePrivate *priv = folder_tree->priv;
-	gchar *full_name = NULL, *uri = NULL;
 	GtkTreeModel *model;
 	GtkTreePath *src_path;
 	CamelFolder *folder;
 	CamelStore *store;
 	GtkTreeIter iter;
+	gchar *folder_name;
+	gchar *folder_uri;
 
 	if (!priv->drag_row || !(src_path =
 		gtk_tree_row_reference_get_path (priv->drag_row)))
@@ -1926,25 +1927,26 @@ tree_drag_data_get (GtkWidget *widget,
 	gtk_tree_model_get (
 		model, &iter,
 		COL_POINTER_CAMEL_STORE, &store,
-		COL_STRING_FULL_NAME, &full_name,
-		COL_STRING_URI, &uri, -1);
+		COL_STRING_FULL_NAME, &folder_name, -1);
 
 	/* make sure user isn't trying to drag on a placeholder row */
-	if (full_name == NULL)
+	if (folder_name == NULL)
 		goto fail;
 
+	folder_uri = e_mail_folder_uri_build (store, folder_name);
+
 	switch (info) {
 	case DND_DRAG_TYPE_FOLDER:
 		/* dragging to a new location in the folder tree */
 		gtk_selection_data_set (
 			selection, drag_atoms[info], 8,
-			(guchar *) uri, strlen (uri) + 1);
+			(guchar *) folder_uri, strlen (folder_uri) + 1);
 		break;
 	case DND_DRAG_TYPE_TEXT_URI_LIST:
 		/* dragging to nautilus or something, probably */
 		/* FIXME camel_store_get_folder_sync() may block. */
 		if ((folder = camel_store_get_folder_sync (
-			store, full_name, 0, NULL, NULL))) {
+			store, folder_name, 0, NULL, NULL))) {
 
 			GPtrArray *uids = camel_folder_get_uids (folder);
 
@@ -1957,10 +1959,85 @@ tree_drag_data_get (GtkWidget *widget,
 		abort ();
 	}
 
+	g_free (folder_uri);
+
 fail:
 	gtk_tree_path_free (src_path);
-	g_free (full_name);
-	g_free (uri);
+	g_free (folder_name);
+}
+
+static gboolean
+ask_drop_folder (EMFolderTree *folder_tree,
+		 const gchar *src_folder_uri,
+		 const gchar *des_full_name,
+		 CamelStore *des_store,
+		 gboolean is_move)
+{
+	const gchar *key = is_move ? "prompt-on-folder-drop-move" : "prompt-on-folder-drop-copy";
+	EMailSession *session;
+	GSettings *settings;
+	gchar *set_value, *src_folder_name = NULL;
+	GError *error = NULL;
+	GtkWidget *widget;
+	GtkWindow *parent;
+	gint response;
+
+	g_return_val_if_fail (folder_tree != NULL, FALSE);
+	g_return_val_if_fail (src_folder_uri != NULL, FALSE);
+	g_return_val_if_fail (des_full_name != NULL || des_store != NULL, FALSE);
+
+	settings = g_settings_new ("org.gnome.evolution.mail");
+	set_value = g_settings_get_string (settings, key);
+
+	if (g_strcmp0 (set_value, "never") == 0) {
+		g_object_unref (settings);
+		g_free (set_value);
+
+		return FALSE;
+	} else if (g_strcmp0 (set_value, "always") == 0) {
+		g_object_unref (settings);
+		g_free (set_value);
+
+		return TRUE;
+	}
+
+	g_free (set_value);
+
+	session = em_folder_tree_get_session (folder_tree);
+	if (!e_mail_folder_uri_parse (CAMEL_SESSION (session),
+				      src_folder_uri, NULL, &src_folder_name, &error)) {
+		g_debug ("%s: Failed to convert '%s' to folder name: %s",
+			G_STRFUNC, src_folder_uri, error ? error->message : "Unknown error");
+		g_clear_error (&error);
+		g_object_unref (settings);
+
+		return FALSE;
+	}
+
+	parent = NULL;
+	widget = gtk_widget_get_toplevel (GTK_WIDGET (folder_tree));
+	if (widget && gtk_widget_is_toplevel (widget) && GTK_IS_WINDOW (widget))
+		parent = GTK_WINDOW (widget);
+
+	widget = e_alert_dialog_new_for_args (
+		parent,
+		is_move ? "mail:ask-folder-drop-move" : "mail:ask-folder-drop-copy",
+		src_folder_name,
+		des_full_name && *des_full_name ? des_full_name :
+			camel_service_get_display_name (CAMEL_SERVICE (des_store)),
+		NULL);
+	response = gtk_dialog_run (GTK_DIALOG (widget));
+	gtk_widget_destroy (widget);
+
+	if (response == GTK_RESPONSE_OK)
+		g_settings_set_string (settings, key, "always");
+	else if (response == GTK_RESPONSE_CANCEL)
+		g_settings_set_string (settings, key, "never");
+
+	g_free (src_folder_name);
+	g_object_unref (settings);
+
+	return response == GTK_RESPONSE_YES || response == GTK_RESPONSE_OK;
 }
 
 /* Drop handling */
@@ -2208,6 +2285,16 @@ tree_drag_data_received (GtkWidget *widget,
 		return;
 	}
 
+	if (info == DND_DROP_TYPE_FOLDER &&
+	    !ask_drop_folder (folder_tree,
+			      (const gchar *) gtk_selection_data_get_data (selection),
+			      full_name, store,
+			      gdk_drag_context_get_selected_action (context) == GDK_ACTION_MOVE)) {
+		gtk_drag_finish (context, FALSE, FALSE, GDK_CURRENT_TIME);
+		gtk_tree_path_free (dest_path);
+		return;
+	}
+
 	m = mail_msg_new (&folder_tree_drop_async_info);
 	m->folder_tree = g_object_ref (folder_tree);
 	m->session = g_object_ref (session);
@@ -2804,10 +2891,23 @@ em_folder_tree_get_selected_uris (EMFolderTree *folder_tree)
 		GtkTreePath *path = l->data;
 
 		if (gtk_tree_model_get_iter (model, &iter, path)) {
-			gchar *uri;
+			CamelStore *store;
+			gchar *folder_name;
 
-			gtk_tree_model_get (model, &iter, COL_STRING_URI, &uri, -1);
-			list = g_list_prepend (list, uri);
+			gtk_tree_model_get (
+				model, &iter,
+				COL_POINTER_CAMEL_STORE, &store,
+				COL_STRING_FULL_NAME, &folder_name, -1);
+
+			if (CAMEL_IS_STORE (store) && folder_name != NULL) {
+				gchar *folder_uri;
+
+				folder_uri = e_mail_folder_uri_build (
+					store, folder_name);
+				list = g_list_prepend (list, folder_uri);
+			}
+
+			g_free (folder_name);
 		}
 		gtk_tree_path_free (path);
 	}
@@ -3256,7 +3356,9 @@ em_folder_tree_get_selected_uri (EMFolderTree *folder_tree)
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gchar *uri = NULL;
+	CamelStore *store;
+	gchar *folder_name;
+	gchar *folder_uri = NULL;
 
 	g_return_val_if_fail (EM_IS_FOLDER_TREE (folder_tree), NULL);
 
@@ -3266,9 +3368,17 @@ em_folder_tree_get_selected_uri (EMFolderTree *folder_tree)
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return NULL;
 
-	gtk_tree_model_get (model, &iter, COL_STRING_URI, &uri, -1);
+	gtk_tree_model_get (
+		model, &iter,
+		COL_POINTER_CAMEL_STORE, &store,
+		COL_STRING_FULL_NAME, &folder_name, -1);
 
-	return uri;
+	if (CAMEL_IS_STORE (store) && folder_name != NULL)
+		folder_uri = e_mail_folder_uri_build (store, folder_name);
+
+	g_free (folder_name);
+
+	return folder_uri;
 }
 
 CamelFolder *
@@ -3356,12 +3466,15 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
                               GKeyFile *key_file)
 {
 	EShell *shell;
+	EMFolderTreeModel *folder_tree_model;
+	EMailSession *session;
 	GtkTreeModel *tree_model;
 	GtkTreeView *tree_view;
 	GtkTreeIter iter;
 	gboolean valid;
 	gchar **groups_arr;
 	GSList *groups, *group;
+	gboolean express_mode;
 	gint ii;
 
 	/* Make sure we have a key file to restore state from. */
@@ -3370,10 +3483,15 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
 
 	/* XXX Pass this in. */
 	shell = e_shell_get_default ();
+	express_mode = e_shell_get_express_mode (shell);
 
 	tree_view = GTK_TREE_VIEW (folder_tree);
 	tree_model = gtk_tree_view_get_model (tree_view);
 
+	folder_tree_model = EM_FOLDER_TREE_MODEL (tree_model);
+	session = em_folder_tree_model_get_session (folder_tree_model);
+	g_return_if_fail (E_IS_MAIL_SESSION (session));
+
 	/* Set the initial folder tree expanded state in two stages:
 	 *
 	 * 1) Iterate over the "Store" and "Folder" state file groups
@@ -3400,39 +3518,66 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
 	groups = g_slist_sort (groups, sort_by_store_and_uri);
 
 	for (group = groups; group != NULL; group = group->next) {
-		GtkTreeRowReference *reference;
-		GtkTreePath *path;
-		GtkTreeIter iter;
+		GtkTreeRowReference *reference = NULL;
+		CamelStore *store = NULL;
 		const gchar *group_name = group->data;
 		const gchar *key = STATE_KEY_EXPANDED;
-		const gchar *uri;
+		gchar *folder_name = NULL;
 		gboolean expanded;
+		gboolean success = FALSE;
 
 		if (g_str_has_prefix (group_name, "Store ")) {
-			uri = group_name + 6;
+			CamelService *service;
+			const gchar *uid = group_name + 6;
+
+			service = camel_session_get_service (
+				CAMEL_SESSION (session), uid);
+			if (CAMEL_IS_STORE (service)) {
+				store = g_object_ref (service);
+				success = TRUE;
+			}
 			expanded = TRUE;
+
 		} else if (g_str_has_prefix (group_name, "Folder ")) {
-			uri = group_name + 7;
+			const gchar *uri = group_name + 7;
+
+			success = e_mail_folder_uri_parse (
+				CAMEL_SESSION (session), uri,
+				&store, &folder_name, NULL);
 			expanded = FALSE;
-		} else
-			continue;
+		}
 
 		if (g_key_file_has_key (key_file, group_name, key, NULL))
 			expanded = g_key_file_get_boolean (
 				key_file, group_name, key, NULL);
 
-		if (!expanded)
-			continue;
+		if (expanded && success) {
+			EMFolderTreeModelStoreInfo *si;
 
-		reference = em_folder_tree_model_lookup_uri (
-			EM_FOLDER_TREE_MODEL (tree_model), uri);
-		if (reference == NULL)
-			continue;
+			si = em_folder_tree_model_lookup_store_info (
+				folder_tree_model, store);
+			if (si != NULL) {
+				if (folder_name != NULL)
+					reference = g_hash_table_lookup (
+						si->full_hash, folder_name);
+				else
+					reference = si->row;
+			}
+		}
 
-		path = gtk_tree_row_reference_get_path (reference);
-		gtk_tree_model_get_iter (tree_model, &iter, path);
-		gtk_tree_view_expand_row (tree_view, path, FALSE);
-		gtk_tree_path_free (path);
+		if (gtk_tree_row_reference_valid (reference)) {
+			GtkTreePath *path;
+			GtkTreeIter iter;
+
+			path = gtk_tree_row_reference_get_path (reference);
+			gtk_tree_model_get_iter (tree_model, &iter, path);
+			gtk_tree_view_expand_row (tree_view, path, FALSE);
+			gtk_tree_path_free (path);
+		}
+
+		if (store != NULL)
+			g_object_unref (store);
+		g_free (folder_name);
 	}
 
 	g_slist_free (groups);
@@ -3443,28 +3588,35 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
 	valid = gtk_tree_model_get_iter_first (tree_model, &iter);
 
 	while (valid) {
+		CamelStore *store;
+		CamelService *service;
 		const gchar *key = STATE_KEY_EXPANDED;
+		const gchar *uid;
 		gboolean expand_row;
+		gboolean built_in_store;
 		gchar *group_name;
-		gchar *uri;
 
 		gtk_tree_model_get (
-			tree_model, &iter, COL_STRING_URI, &uri, -1);
+			tree_model, &iter,
+			COL_POINTER_CAMEL_STORE, &store, -1);
 
-		if (uri == NULL)
+		if (!CAMEL_IS_STORE (store))
 			goto next;
 
-		group_name = g_strdup_printf ("Store %s", uri);
+		service = CAMEL_SERVICE (store);
+		uid = camel_service_get_uid (service);
+		group_name = g_strdup_printf ("Store %s", uid);
 
 		/* Expand stores that have no "Expanded" key. */
 		expand_row = !g_key_file_has_key (
 			key_file, group_name, key, NULL);
 
-		/* Do not expand local stores in Express mode. */
-		if (e_shell_get_express_mode (shell)) {
-			expand_row &= (strncmp (uri, "vfolder", 7) != 0);
-			expand_row &= (strncmp (uri, "maildir", 7) != 0);
-		}
+		built_in_store =
+			(g_strcmp0 (uid, E_MAIL_SESSION_LOCAL_UID) == 0) ||
+			(g_strcmp0 (uid, E_MAIL_SESSION_VFOLDER_UID) == 0);
+
+		if (express_mode && built_in_store)
+			expand_row = FALSE;
 
 		if (expand_row) {
 			GtkTreePath *path;
@@ -3475,7 +3627,6 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
 		}
 
 		g_free (group_name);
-		g_free (uri);
 
 	next:
 		valid = gtk_tree_model_iter_next (tree_model, &iter);
