@@ -844,7 +844,8 @@ receive_status (CamelFilterDriver *driver,
 
 /* when receive/send is complete */
 static void
-receive_done (int still_more, gpointer data)
+receive_done (gint still_more,
+              gpointer data)
 {
 	struct _send_info *info = data;
 	const gchar *uid;
@@ -897,21 +898,10 @@ receive_done (int still_more, gpointer data)
 
 	/* remove/free this active download */
 	d(printf("%s: freeing info %p\n", G_STRFUNC, info));
-	if (info->type == SEND_SEND) {
-		gpointer key = NULL, value = NULL;
-		if (!g_hash_table_lookup_extended (info->data->active, SEND_URI_KEY, &key, &value))
-			key = NULL;
-
+	if (info->type == SEND_SEND)
 		g_hash_table_steal (info->data->active, SEND_URI_KEY);
-		g_free (key);
-	} else {
-		gpointer key = NULL, value = NULL;
-		if (!g_hash_table_lookup_extended (info->data->active, uid, &key, &value))
-			key = NULL;
-
+	else
 		g_hash_table_steal (info->data->active, uid);
-		g_free (key);
-	}
 	info->data->infos = g_list_remove (info->data->infos, info);
 
 	if (g_hash_table_size (info->data->active) == 0) {
@@ -1051,19 +1041,17 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 			m->folders->pdata[i], 0,
 			cancellable, &local_error);
 		if (folder) {
-			if (camel_folder_synchronize_sync (folder, FALSE, cancellable, &local_error))
-				camel_folder_refresh_info_sync (folder, cancellable, &local_error);
+			/* FIXME Not passing a GError here. */
+			camel_folder_synchronize_sync (
+				folder, FALSE, cancellable, NULL);
+			camel_folder_refresh_info_sync (folder, cancellable, NULL);
 			g_object_unref (folder);
-		}
-
-		if (local_error != NULL) {
-			if (!g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-				g_warning ("Failed to refresh folders: %s", local_error->message);
+		} else if (local_error != NULL) {
+			g_warning ("Failed to refresh folders: %s", local_error->message);
 			g_clear_error (&local_error);
 		}
 
-		if (g_cancellable_is_cancelled (m->info->cancellable) ||
-		    g_cancellable_is_cancelled (cancellable))
+		if (g_cancellable_is_cancelled (m->info->cancellable))
 			break;
 
 		if (m->info->state != SEND_CANCELLED)
@@ -1485,7 +1473,7 @@ mail_receive_service (CamelService *service)
 	case SEND_RECEIVE:
 		mail_fetch_mail (
 			CAMEL_STORE (service),
-			info->keep_on_server, 0, -1, 
+			info->keep_on_server, 0, -1,
 			E_FILTER_SOURCE_INCOMING,
 			NULL, NULL, NULL,
 			info->cancellable,
