@@ -176,9 +176,9 @@ create_calendar_sources (EShellBackend *shell_backend,
 
 			e_shell_settings_set_string (
 				shell_settings, "cal-primary-calendar",
-				e_source_peek_uid (source));
+				e_source_get_uid (source));
 
-			link.data = (gpointer) e_source_peek_uid (source);
+			link.data = (gpointer) e_source_get_uid (source);
 			link.next = NULL;
 
 			e_cal_shell_backend_set_selected_calendars (
