@@ -46,13 +46,69 @@ cal_shell_view_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+cal_shell_view_add_action_button (GtkBox *box,
+				  GtkAction *action)
+{
+	GtkWidget *button, *icon;
+
+	g_return_if_fail (box != NULL);
+	g_return_if_fail (action != NULL);
+
+	icon = gtk_action_create_icon (action, GTK_ICON_SIZE_BUTTON);
+	button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (button), icon);
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+
+	g_object_bind_property (
+		action, "visible",
+		button, "visible",
+		G_BINDING_SYNC_CREATE);
+
+	g_object_bind_property (
+		action, "sensitive",
+		button, "sensitive",
+		G_BINDING_SYNC_CREATE);
+
+	g_object_bind_property (
+		action, "tooltip",
+		button, "tooltip-text",
+		G_BINDING_SYNC_CREATE);
+
+	g_signal_connect_swapped (button, "clicked",
+		G_CALLBACK (gtk_action_activate), action);
+}
+
 static void
 cal_shell_view_constructed (GObject *object)
 {
+	EShellContent *shell_content;
+	EShellWindow *shell_window;
+	EShellSearchbar *searchbar;
+	GtkWidget *box;
+
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (parent_class)->constructed (object);
 
 	e_cal_shell_view_private_constructed (E_CAL_SHELL_VIEW (object));
+
+	/* no search bar in express mode */
+	if (e_shell_get_express_mode (e_shell_get_default ()))
+		return;
+
+	shell_window = e_shell_view_get_shell_window (E_SHELL_VIEW (object));
+	shell_content = e_shell_view_get_shell_content (E_SHELL_VIEW (object));
+	searchbar = e_cal_shell_content_get_searchbar (E_CAL_SHELL_CONTENT (shell_content));	
+
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+
+	cal_shell_view_add_action_button (GTK_BOX (box), ACTION (CALENDAR_SEARCH_PREV));
+	cal_shell_view_add_action_button (GTK_BOX (box), ACTION (CALENDAR_SEARCH_NEXT));
+	cal_shell_view_add_action_button (GTK_BOX (box), ACTION (CALENDAR_SEARCH_STOP));
+
+	gtk_widget_show_all (box);
+
+	gtk_box_pack_start (GTK_BOX (e_shell_searchbar_get_search_box (searchbar)), box, FALSE, FALSE, 0);
 }
 
 static void
@@ -81,6 +137,8 @@ cal_shell_view_execute_search (EShellView *shell_view)
 	gchar *temp;
 	gint value;
 
+	e_cal_shell_view_search_stop (E_CAL_SHELL_VIEW (shell_view));
+
 	shell_window = e_shell_view_get_shell_window (shell_view);
 	shell_content = e_shell_view_get_shell_content (shell_view);
 	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
@@ -239,6 +297,9 @@ cal_shell_view_execute_search (EShellView *shell_view)
 	gnome_calendar_set_search_query (
 		calendar, query, range_search, start_range, end_range);
 	g_free (query);
+
+	/* update also actions, thus Find Prev/Next/Stop will be sensitive as expected */
+	e_shell_view_update_actions (shell_view);
 }
 
 static icalproperty *
@@ -318,9 +379,12 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	EShell *shell;
 	GnomeCalendarViewType view_type;
 	GnomeCalendar *calendar;
+	ECalModel *model;
 	ECalendarView *view;
 	GtkAction *action;
 	GList *list, *iter;
+	const gchar *model_sexp;
+	gboolean is_searching;
 	gboolean sensitive;
 	gboolean visible;
 	guint32 state;
@@ -364,6 +428,11 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
 	view_type = gnome_calendar_get_view (calendar);
 	view = gnome_calendar_get_calendar_view (calendar, view_type);
+	model = gnome_calendar_get_model (calendar);
+	model_sexp = e_cal_model_get_search_query (model);
+	is_searching = model_sexp && *model_sexp &&
+		g_strcmp0 (model_sexp, "#t") != 0 &&
+		g_strcmp0 (model_sexp, "(contains? \"summary\"  \"\")") != 0;
 
 	list = e_calendar_view_get_selected_events (view);
 	n_selected = g_list_length (list);
@@ -452,6 +521,15 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	sensitive = can_delete_primary_source;
 	gtk_action_set_sensitive (action, sensitive);
 
+	action = ACTION (CALENDAR_SEARCH_PREV);
+	gtk_action_set_sensitive (action, is_searching);
+
+	action = ACTION (CALENDAR_SEARCH_NEXT);
+	gtk_action_set_sensitive (action, is_searching);
+
+	action = ACTION (CALENDAR_SEARCH_STOP);
+	gtk_action_set_sensitive (action, is_searching && priv->searching_activity != NULL);
+
 	action = ACTION (EVENT_DELEGATE);
 	sensitive =
 		(n_selected == 1) && editable &&
@@ -511,7 +589,7 @@ cal_shell_view_update_actions (EShellView *shell_view)
 
 static void
 cal_shell_view_class_init (ECalShellViewClass *class,
-                            GTypeModule *type_module)
+                           GTypeModule *type_module)
 {
 	GObjectClass *object_class;
 	EShellViewClass *shell_view_class;
