@@ -16,11 +16,16 @@
  *
  */
 
-#include "camel-sasl-xoauth.h"
-
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <glib/gi18n-lib.h>
 
+#include <libemail-utils/e-account-utils.h>
+
+#include "camel-sasl-xoauth.h"
+
 #define GOA_API_IS_SUBJECT_TO_CHANGE
 #include <goa/goa.h>
 
@@ -73,7 +78,7 @@ hmac_sha1 (const gchar *key,
 	guchar inner[SHA1_LENGTH];
 	guchar digest[SHA1_LENGTH];
 	gsize key_length, inner_length, digest_length;
-	int i;
+	gint i;
 
 	g_return_val_if_fail (key, NULL);
 	g_return_val_if_fail (message, NULL);
@@ -86,7 +91,7 @@ hmac_sha1 (const gchar *key,
 
 		key_length = sizeof (new_key);
 
-		g_checksum_update (checksum, (guchar*)key, strlen (key));
+		g_checksum_update (checksum, (guchar *) key, strlen (key));
 		g_checksum_get_digest (checksum, new_key, &key_length);
 		g_checksum_reset (checksum);
 
@@ -117,7 +122,7 @@ hmac_sha1 (const gchar *key,
 
 	/* Stage 3 and 4 */
 	g_checksum_update (checksum, ipad, sizeof (ipad));
-	g_checksum_update (checksum, (guchar*)message, strlen (message));
+	g_checksum_update (checksum, (guchar *) message, strlen (message));
 	inner_length = sizeof (inner);
 	g_checksum_get_digest (checksum, inner, &inner_length);
 	g_checksum_reset (checksum);
@@ -135,7 +140,7 @@ hmac_sha1 (const gchar *key,
 	return g_base64_encode (digest, digest_length);
 }
 
-static char *
+static gchar *
 sign_plaintext (const gchar *consumer_secret,
                 const gchar *token_secret)
 {
@@ -153,7 +158,7 @@ sign_plaintext (const gchar *consumer_secret,
 	return rv;
 }
 
-static char *
+static gchar *
 sign_hmac (const gchar *consumer_secret,
            const gchar *token_secret,
            const gchar *http_method,
@@ -352,21 +357,30 @@ sasl_xoauth_challenge_sync (CamelSasl *sasl,
 	GoaAccount *goa_account;
 	GByteArray *parameters = NULL;
 	CamelService *service;
+	EAccount *account;
 	CamelURL *url;
 	const gchar *account_id;
+	const gchar *uid;
 	gchar *xoauth_param = NULL;
 	gboolean success;
 
 	service = camel_sasl_get_service (sasl);
-	url = camel_service_get_camel_url (service);
-	account_id = camel_url_get_param (url, GOA_KEY);
-	g_return_val_if_fail (account_id != NULL, NULL);
+	uid = camel_service_get_uid (service);
+	account = e_get_account_by_uid (uid);
+	g_return_val_if_fail (account != NULL, NULL);
 
 	goa_client = goa_client_new_sync (cancellable, error);
 	if (goa_client == NULL)
 		return NULL;
 
+	url = camel_url_new (account->source->url, error);
+	if (url == NULL)
+		return NULL;
+
+	account_id = camel_url_get_param (url, GOA_KEY);
 	goa_object = sasl_xoauth_get_account_by_id (goa_client, account_id);
+	camel_url_free (url);
+
 	if (goa_object == NULL) {
 		g_set_error_literal (
 			error, CAMEL_SERVICE_ERROR,
