@@ -26,6 +26,7 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 #include <libedataserver/e-source.h>
 #include <libedataserverui/e-client-utils.h>
 #include <libedataserverui/e-source-selector.h>
@@ -201,14 +202,14 @@ do_save_calendar_rdf (FormatHandler *handler,
 	if (!dest_uri)
 		return;
 
-	primary_source = e_source_selector_get_primary_selection (selector);
-
 	/* open source client */
+	primary_source = e_source_selector_ref_primary_selection (selector);
 	source_client = e_cal_client_new (primary_source, type, &error);
 	if (source_client)
 		g_signal_connect (
 			source_client, "authenticate",
 			G_CALLBACK (e_client_utils_authenticate_handler), NULL);
+	g_object_unref (primary_source);
 
 	if (!source_client || !e_client_open_sync (E_CLIENT (source_client), TRUE, NULL, &error)) {
 		display_error_message (gtk_widget_get_toplevel (GTK_WIDGET (selector)), error);
@@ -249,9 +250,9 @@ do_save_calendar_rdf (FormatHandler *handler,
 
 		xmlNewChild (fnode, NULL, (const guchar *)"method", (const guchar *)"PUBLISH");
 
-		xmlNewChild (fnode, NULL, (const guchar *)"x-wr:relcalid", (guchar *)e_source_peek_uid (primary_source));
+		xmlNewChild (fnode, NULL, (const guchar *)"x-wr:relcalid", (guchar *)e_source_get_uid (primary_source));
 
-		xmlNewChild (fnode, NULL, (const guchar *)"x-wr:calname", (guchar *)e_source_peek_name (primary_source));
+		xmlNewChild (fnode, NULL, (const guchar *)"x-wr:calname", (guchar *)e_source_get_display_name (primary_source));
 
 		/* Version of this RDF-format */
 		xmlNewChild (fnode, NULL, (const guchar *)"version", (const guchar *)"2.0");
