@@ -67,11 +67,10 @@ e_shell_configure_ui_manager (EShell *shell,
  * @customize_data: optional data to pass to @customize_func
  *
  * Runs a #GtkFileChooserDialog in open mode with the given title and
- * returns the selected #GFile.  It automatically remembers the selected
- * folder.  If @customize_func is provided, the function is called just
- * prior to running the dialog (the file chooser is the first argument,
- * @customize data is the second).  If the user cancels the dialog the
- * function will return %NULL.
+ * returns the selected #GFile.  If @customize_func is provided, the
+ * function is called just prior to running the dialog (the file chooser
+ * is the first argument, @customize data is the second).  If the user
+ * cancels the dialog the function will return %NULL.
  *
  * Returns: the #GFile to open, or %NULL
  **/
@@ -81,19 +80,13 @@ e_shell_run_open_dialog (EShell *shell,
                          GtkCallback customize_func,
                          gpointer customize_data)
 {
-	EShellSettings *shell_settings;
 	GtkFileChooser *file_chooser;
 	GFile *chosen_file = NULL;
 	GtkWidget *dialog;
 	GtkWindow *parent;
-	const gchar *property_name;
-	gchar *uri;
 
 	g_return_val_if_fail (E_IS_SHELL (shell), NULL);
 
-	property_name = "file-chooser-folder";
-	shell_settings = e_shell_get_shell_settings (shell);
-
 	parent = e_shell_get_active_window (shell);
 
 	dialog = gtk_file_chooser_dialog_new (
@@ -109,27 +102,13 @@ e_shell_run_open_dialog (EShell *shell,
 
 	gtk_file_chooser_set_local_only (file_chooser, FALSE);
 
-	/* Restore the current folder from the previous file chooser. */
-	uri = e_shell_settings_get_string (shell_settings, property_name);
-	if (uri != NULL)
-		gtk_file_chooser_set_current_folder_uri (file_chooser, uri);
-	g_free (uri);
-
 	/* Allow further customizations before running the dialog. */
 	if (customize_func != NULL)
 		customize_func (dialog, customize_data);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_ACCEPT)
-		goto exit;
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+		chosen_file = gtk_file_chooser_get_file (file_chooser);
 
-	chosen_file = gtk_file_chooser_get_file (file_chooser);
-
-	/* Save the current folder for subsequent file choosers. */
-	uri = gtk_file_chooser_get_current_folder_uri (file_chooser);
-	e_shell_settings_set_string (shell_settings, property_name, uri);
-	g_free (uri);
-
-exit:
 	gtk_widget_destroy (dialog);
 
 	return chosen_file;
@@ -145,16 +124,15 @@ exit:
  * @customize_data: optional data to pass to @customize_func
  *
  * Runs a #GtkFileChooserDialog in save mode with the given title and
- * returns the selected #GFile.  It automatically remembers the selected
- * folder.  If @customize_func is provided, the function is called just
- * prior to running the dialog (the file chooser is the first argument,
- * @customize_data is the second).  If the user cancels the dialog the
- * function will return %NULL.
+ * returns the selected #GFile.  If @customize_func is provided, the
+ * function is called just prior to running the dialog (the file chooser
+ * is the first argument, @customize_data is the second).  If the user
+ * cancels the dialog the function will return %NULL.
  *
  * With non-%NULL @filters will be added also file filters to the dialog.
  * The string format is "pat1:mt1;pat2:mt2:...", where 'pat' is a pattern
- * and 'mt' is a MIME type for the pattern to be used.
- * There can be more than one MIME type, those are separated by comma.
+ * and 'mt' is a MIME type for the pattern to be used.  There can be more
+ * than one MIME type, those are separated by comma.
  *
  * Returns: the #GFile to save to, or %NULL
  **/
@@ -166,19 +144,13 @@ e_shell_run_save_dialog (EShell *shell,
                          GtkCallback customize_func,
                          gpointer customize_data)
 {
-	EShellSettings *shell_settings;
 	GtkFileChooser *file_chooser;
 	GFile *chosen_file = NULL;
 	GtkWidget *dialog;
 	GtkWindow *parent;
-	const gchar *property_name;
-	gchar *uri;
 
 	g_return_val_if_fail (E_IS_SHELL (shell), NULL);
 
-	property_name = "file-chooser-folder";
-	shell_settings = e_shell_get_shell_settings (shell);
-
 	parent = e_shell_get_active_window (shell);
 
 	dialog = gtk_file_chooser_dialog_new (
@@ -195,12 +167,6 @@ e_shell_run_save_dialog (EShell *shell,
 	gtk_file_chooser_set_local_only (file_chooser, FALSE);
 	gtk_file_chooser_set_do_overwrite_confirmation (file_chooser, TRUE);
 
-	/* Restore the current folder from the previous file chooser. */
-	uri = e_shell_settings_get_string (shell_settings, property_name);
-	if (uri != NULL)
-		gtk_file_chooser_set_current_folder_uri (file_chooser, uri);
-	g_free (uri);
-
 	if (suggestion != NULL) {
 		gchar *current_name;
 
@@ -262,17 +228,9 @@ e_shell_run_save_dialog (EShell *shell,
 	if (customize_func != NULL)
 		customize_func (dialog, customize_data);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_ACCEPT)
-		goto exit;
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+		chosen_file = gtk_file_chooser_get_file (file_chooser);
 
-	chosen_file = gtk_file_chooser_get_file (file_chooser);
-
-	/* Save the current folder for subsequent file choosers. */
-	uri = gtk_file_chooser_get_current_folder_uri (file_chooser);
-	e_shell_settings_set_string (shell_settings, property_name, uri);
-	g_free (uri);
-
-exit:
 	gtk_widget_destroy (dialog);
 
 	return chosen_file;
@@ -314,7 +272,9 @@ e_shell_utils_import_uris (EShell *shell,
 			assistant, "finished",
 			G_CALLBACK (gtk_widget_destroy), NULL);
 
-		e_shell_watch_window (shell, GTK_WINDOW (assistant));
+		gtk_application_add_window (
+			GTK_APPLICATION (shell),
+			GTK_WINDOW (assistant));
 
 		gtk_widget_show (assistant);
 	} else
