@@ -118,9 +118,6 @@ static gchar *geometry = NULL;
 static gchar *requested_view = NULL;
 static gchar **remaining_args;
 
-/* Forward declarations */
-void e_convert_local_mail (EShell *shell);
-
 static void
 categories_icon_theme_hack (void)
 {
@@ -200,9 +197,9 @@ show_development_warning (void)
 		  "this version, and install version %s instead.\n"
 		  "\n"
 		  "If you find bugs, please report them to us at bugzilla.gnome.org.\n"
-                  "This product comes with no warranty and is not intended for\n"
+		  "This product comes with no warranty and is not intended for\n"
 		  "individuals prone to violent fits of anger.\n"
-                  "\n"
+		  "\n"
 		  "We hope that you enjoy the results of our hard work, and we\n"
 		  "eagerly await your contributions!\n"),
 		STABLE_VERSION);
@@ -654,18 +651,6 @@ main (gint argc,
 		goto exit;
 	}
 
-	/* This routine converts the local mail store from mbox format to
-	 * Maildir format as needed.  The reason the code is here and not
-	 * in the mail module is because we inform the user at startup of
-	 * the impending mail conversion by displaying a popup dialog and
-	 * waiting for confirmation before proceeding.
-	 *
-	 * This has to be done before we load modules because some of the
-	 * EShellBackends immediately add GMainContext sources that would
-	 * otherwise get dispatched during gtk_dialog_run(), and we don't
-	 * want them dispatched until after the conversion is complete. */
-	e_convert_local_mail (shell);
-
 	e_shell_load_modules (shell);
 
 	if (!disable_eplugin) {
