@@ -1454,7 +1454,6 @@ typedef struct _LoadContext LoadContext;
 
 struct _LoadContext {
 	EAttachment *attachment;
-	CamelMimePart *mime_part;
 	GSimpleAsyncResult *simple;
 
 	GInputStream *input_stream;
@@ -1496,12 +1495,7 @@ static void
 attachment_load_context_free (LoadContext *load_context)
 {
 	g_object_unref (load_context->attachment);
-
-	if (load_context->mime_part != NULL)
-		g_object_unref (load_context->mime_part);
-
-	if (load_context->simple)
-		g_object_unref (load_context->simple);
+	g_object_unref (load_context->simple);
 
 	if (load_context->input_stream != NULL)
 		g_object_unref (load_context->input_stream);
@@ -1596,19 +1590,17 @@ attachment_load_finish (LoadContext *load_context)
 		camel_mime_part_set_disposition (mime_part, disposition);
 
 	/* Correctly report the size of zero length special files. */
-	if (g_file_info_get_size (file_info) == 0)
+	if (g_file_info_get_size (file_info) == 0) {
 		g_file_info_set_size (file_info, size);
-
-	load_context->mime_part = mime_part;
+		e_attachment_set_file_info (attachment, file_info);
+	}
 
 	g_simple_async_result_set_op_res_gpointer (
-		simple, load_context, (GDestroyNotify) attachment_load_context_free);
+		simple, mime_part, (GDestroyNotify) g_object_unref);
 
 	g_simple_async_result_complete (simple);
 
-	/* make sure it's freed on operation end */
-	load_context->simple = NULL;
-	g_object_unref (simple);
+	attachment_load_context_free (load_context);
 }
 
 static void
@@ -1844,15 +1836,18 @@ attachment_load_from_mime_part_thread (GSimpleAsyncResult *simple,
 	g_file_info_set_size (file_info, CAMEL_STREAM_NULL (null)->written);
 	g_object_unref (null);
 
-	load_context->mime_part = g_object_ref (mime_part);
+	string = camel_mime_part_get_disposition (mime_part);
+	e_attachment_set_disposition (attachment, string);
 
-	/* make sure it's freed on operation end */
-	g_object_unref (load_context->simple);
-	load_context->simple = NULL;
+	e_attachment_set_file_info (attachment, file_info);
+
+	g_object_ref (mime_part);
 
 	g_simple_async_result_set_op_res_gpointer (
-		simple, load_context,
-		(GDestroyNotify) attachment_load_context_free);
+		simple, mime_part,
+		(GDestroyNotify) g_object_unref);
+
+	attachment_load_context_free (load_context);
 }
 
 void
@@ -1916,28 +1911,20 @@ e_attachment_load_finish (EAttachment *attachment,
                           GError **error)
 {
 	GSimpleAsyncResult *simple;
-	const LoadContext *load_context;
+	CamelMimePart *mime_part;
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), FALSE);
 	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (result), FALSE);
 
 	simple = G_SIMPLE_ASYNC_RESULT (result);
-	load_context = g_simple_async_result_get_op_res_gpointer (simple);
-	if (load_context && load_context->mime_part) {
-		const gchar *string;
-
-		string = camel_mime_part_get_disposition (load_context->mime_part);
-		e_attachment_set_disposition (attachment, string);
-
-		e_attachment_set_file_info (attachment, load_context->file_info);
-		e_attachment_set_mime_part (attachment, load_context->mime_part);
-	}
-
+	mime_part = g_simple_async_result_get_op_res_gpointer (simple);
+	if (mime_part != NULL)
+		e_attachment_set_mime_part (attachment, mime_part);
 	g_simple_async_result_propagate_error (simple, error);
 
 	attachment_set_loading (attachment, FALSE);
 
-	return (load_context != NULL);
+	return (mime_part != NULL);
 }
 
 void
@@ -1954,7 +1941,7 @@ e_attachment_load_handle_error (EAttachment *attachment,
 
 	g_return_if_fail (E_IS_ATTACHMENT (attachment));
 	g_return_if_fail (G_IS_ASYNC_RESULT (result));
-	g_return_if_fail (GTK_IS_WINDOW (parent));
+	g_return_if_fail (!parent || GTK_IS_WINDOW (parent));
 
 	if (e_attachment_load_finish (attachment, result, &error))
 		return;
@@ -1972,8 +1959,10 @@ e_attachment_load_handle_error (EAttachment *attachment,
 	}
 
 	/* Ignore cancellations. */
-	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_error_free (error);
 		return;
+	}
 
 	file_info = e_attachment_get_file_info (attachment);
 
